# -*- coding: utf-8 -*-
"""
AI-Ops Monitor SDK
智能日志监控与自动修复 Python SDK
"""

__version__ = "0.1.2"
__author__ = "Tuple"

from .client import MonitorClient
from .config import MonitorConfig
from .handlers import MonitoringHandler

# 便捷初始化函数
def init_monitor(app=None, **kwargs):
    """
    初始化监控
    
    Args:
        app: 应用实例（Flask/FastAPI/Django 等）
        **kwargs: 配置参数
        
    Examples:
        # Flask
        >>> from flask import Flask
        >>> app = Flask(__name__)
        >>> init_monitor(app, api_url='...', api_key='...')
        
        # FastAPI
        >>> from fastapi import FastAPI
        >>> app = FastAPI()
        >>> init_monitor(app, api_url='...', api_key='...')
        
        # 独立使用
        >>> init_monitor(api_url='...', api_key='...')
    """
    config = MonitorConfig(**kwargs)
    client = MonitorClient(config)
    
    if app is None:
        # 独立使用，添加到 root logger
        import logging
        handler = MonitoringHandler(client, config)
        logging.root.addHandler(handler)
        return client
    
    # 检测框架类型并集成
    app_type = type(app).__module__
    
    if 'flask' in app_type.lower():
        from .integrations.flask import setup_flask
        return setup_flask(app, client, config)
    elif 'fastapi' in app_type.lower() or 'starlette' in app_type.lower():
        from .integrations.fastapi import setup_fastapi
        return setup_fastapi(app, client, config)
    else:
        raise ValueError(f"Unsupported framework: {app_type}")

__all__ = [
    'MonitorClient',
    'MonitorConfig', 
    'MonitoringHandler',
    'init_monitor',
]



# -*- coding: utf-8 -*-
"""
API 客户端模块
"""

import json
import logging
import traceback as tb
from datetime import datetime
from typing import Optional, Dict, Any

import requests
from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

from .config import MonitorConfig

logger = logging.getLogger(__name__)


class MonitorClient:
    """监控 API 客户端"""
    
    def __init__(self, config: MonitorConfig):
        self.config = config
        self.session = self._create_session()
    
    def _create_session(self) -> requests.Session:
        """创建 requests session 并配置重试"""
        session = requests.Session()
        
        # 配置重试策略
        retry_strategy = Retry(
            total=self.config.max_retries,
            backoff_factor=self.config.retry_delay,
            status_forcelist=[429, 500, 502, 503, 504],
            allowed_methods=["POST"]
        )
        
        adapter = HTTPAdapter(max_retries=retry_strategy)
        session.mount("http://", adapter)
        session.mount("https://", adapter)
        
        # 设置默认 headers
        session.headers.update({
            'Content-Type': 'application/json',
            'X-API-Key': self.config.api_key,
            'User-Agent': f'aiops-monitor/0.1.0'
        })
        
        return session
    
    def send_log(
        self,
        level: str,
        message: str,
        traceback: Optional[str] = None,
        context: Optional[Dict[str, Any]] = None
    ) -> bool:
        """
        发送日志到监控系统
        
        Args:
            level: 日志级别
            message: 日志消息
            traceback: 异常堆栈
            context: 额外上下文
            
        Returns:
            是否发送成功
        """
        if not self.config.enabled:
            return False
        
        try:
            url = f"{self.config.api_url}/logs"
            
            payload = {
                'level': level.lower(),
                'message': message,
                'timestamp': datetime.utcnow().isoformat(),
            }
            
            if traceback:
                payload['traceback'] = traceback
            
            if context:
                payload['context'] = context
            
            response = self.session.post(
                url,
                json=payload,
                timeout=self.config.timeout
            )
            
            if response.status_code == 200:
                return True
            else:
                logger.warning(
                    f"监控日志发送失败: {response.status_code} - {response.text}"
                )
                return False
                
        except Exception as e:
            logger.error(f"监控日志发送异常: {e}")
            return False
    
    def format_log_data(
        self,
        record: logging.LogRecord
    ) -> Dict[str, Any]:
        """
        格式化日志记录
        
        Args:
            record: logging.LogRecord 对象
            
        Returns:
            格式化后的日志数据
        """
        data = {
            'level': record.levelname.lower(),
            'message': record.getMessage(),
            'timestamp': datetime.utcnow().isoformat(),
            'context': {
                'logger': record.name,
                'module': record.module,
                'function': record.funcName,
                'line': record.lineno,
                'thread': record.thread,
                'process': record.process,
            }
        }
        
        # 添加项目名称（如果配置了）
        if self.config.project_name:
            data['context']['project'] = self.config.project_name
        
        # 提取异常信息
        if record.exc_info:
            data['traceback'] = ''.join(
                tb.format_exception(*record.exc_info)
            )
        
        return data
    
    def close(self):
        """关闭 session"""
        if self.session:
            self.session.close()



# -*- coding: utf-8 -*-
"""
日志处理器模块
"""

import logging
import threading
from queue import Queue, Empty
from typing import Optional

from .client import MonitorClient
from .config import MonitorConfig


class MonitoringHandler(logging.Handler):
    """
    监控系统日志处理器
    异步发送日志到监控系统
    """
    
    def __init__(
        self,
        client: Optional[MonitorClient] = None,
        config: Optional[MonitorConfig] = None,
        level: int = logging.ERROR
    ):
        super().__init__(level)
        self.client = client
        self.config = config
        self.enabled = config.enabled if config else False
        
        # 异步队列
        if config and config.async_mode:
            self.queue = Queue(maxsize=1000)
            self.worker_thread = threading.Thread(
                target=self._worker,
                daemon=True,
                name='MonitorWorker'
            )
            self.worker_thread.start()
        else:
            self.queue = None
            self.worker_thread = None
    
    def emit(self, record: logging.LogRecord):
        """
        处理日志记录
        
        Args:
            record: 日志记录对象
        """
        if not self.config or not self.client:
            return

        try:
            # 检查是否启用
            if not self.config.enabled:
                return
            
            # 检查日志级别
            if record.levelname not in self.config.log_levels:
                return
            
            # 自定义过滤器
            if self.config.filter_func and not self.config.filter_func(record):
                return
            
            # 忽略特定模式
            if self._should_ignore(record):
                return
            
            # 格式化日志数据
            log_data = self.client.format_log_data(record)
            
            # 异步或同步发送
            if self.config.async_mode and self.queue:
                try:
                    self.queue.put_nowait(log_data)
                except:
                    # 队列满，丢弃日志
                    pass
            else:
                self._send_log(log_data)
                
        except Exception:
            self.handleError(record)
    
    def _should_ignore(self, record: logging.LogRecord) -> bool:
        """检查是否应该忽略该日志"""
        message = record.getMessage()
        for pattern in self.config.ignore_patterns:
            if pattern in message:
                return True
        return False
    
    def _worker(self):
        """后台工作线程，异步发送日志"""
        while True:
            try:
                log_data = self.queue.get(timeout=1.0)
                self._send_log(log_data)
                self.queue.task_done()
            except Empty:
                continue
            except Exception as e:
                logging.error(f"监控 worker 异常: {e}")
    
    def _send_log(self, log_data: dict):
        """发送日志"""
        try:
            self.client.send_log(
                level=log_data['level'],
                message=log_data['message'],
                traceback=log_data.get('traceback'),
                context=log_data.get('context')
            )
        except Exception as e:
            logging.error(f"发送监控日志失败: {e}")
    
    def close(self):
        """关闭处理器"""
        if self.queue and self.worker_thread:
            # 等待队列清空
            self.queue.join()
        
        if self.client:
            self.client.close()
        
        super().close()



# -*- coding: utf-8 -*-
"""
配置管理模块
"""

from typing import List, Optional
from dataclasses import dataclass, field


@dataclass
class MonitorConfig:
    """监控配置类"""
    
    # 必需配置
    api_url: str
    api_key: str
    
    # 可选配置
    project_name: Optional[str] = None
    log_levels: List[str] = field(default_factory=lambda: ['ERROR', 'CRITICAL'])
    timeout: int = 5
    async_mode: bool = True
    
    # 重试配置
    max_retries: int = 3
    retry_delay: float = 1.0
    
    # 高级配置
    enabled: bool = True
    filter_func: Optional[callable] = None
    ignore_patterns: List[str] = field(default_factory=list)
    
    def __post_init__(self):
        """验证配置"""
        if not self.api_url:
            raise ValueError("api_url is required")
        if not self.api_key:
            raise ValueError("api_key is required")
        
        # 规范化 API URL
        self.api_url = self.api_url.rstrip('/')
        
        # 转换日志级别为大写
        self.log_levels = [level.upper() for level in self.log_levels]
    
    @classmethod
    def from_dict(cls, config_dict: dict) -> 'MonitorConfig':
        """从字典创建配置"""
        return cls(**config_dict)
    
    def to_dict(self) -> dict:
        """转换为字典"""
        return {
            'api_url': self.api_url,
            'api_key': self.api_key,
            'project_name': self.project_name,
            'log_levels': self.log_levels,
            'timeout': self.timeout,
            'async_mode': self.async_mode,
            'max_retries': self.max_retries,
            'retry_delay': self.retry_delay,
            'enabled': self.enabled,
        }



# -*- coding: utf-8 -*-
"""
Flask 集成模块
"""

import logging
from flask import Flask

from ..client import MonitorClient
from ..config import MonitorConfig
from ..handlers import MonitoringHandler


def setup_flask(app: Flask, client: MonitorClient, config: MonitorConfig):
    """
    设置 Flask 应用的监控
    
    Args:
        app: Flask 应用实例
        client: 监控客户端
        config: 监控配置
    """
    # 创建 handler
    handler = MonitoringHandler(
        client=client,
        config=config,
        level=getattr(logging, config.log_levels[0])
    )
    
    # 添加到 Flask logger
    app.logger.addHandler(handler)
    
    # 也添加到 root logger（捕获其他模块的日志）
    logging.root.addHandler(handler)
    
    # 关闭时清理
    @app.teardown_appcontext
    def cleanup(error=None):
        handler.close()
    
    return client



# -*- coding: utf-8 -*-
"""
FastAPI 集成模块
"""

import logging
from fastapi import FastAPI

from ..client import MonitorClient
from ..config import MonitorConfig
from ..handlers import MonitoringHandler


def setup_fastapi(app: FastAPI, client: MonitorClient, config: MonitorConfig):
    """
    设置 FastAPI 应用的监控
    
    Args:
        app: FastAPI 应用实例
        client: 监控客户端
        config: 监控配置
    """
    # 创建 handler
    handler = MonitoringHandler(
        client=client,
        config=config,
        level=getattr(logging, config.log_levels[0])
    )
    
    # 添加到 root logger
    logging.root.addHandler(handler)
    
    # 启动和关闭事件
    @app.on_event("startup")
    async def startup():
        pass
    
    @app.on_event("shutdown")
    async def shutdown():
        handler.close()
    
    return client



# -*- coding: utf-8 -*-
"""
Django 集成模块
"""

import logging
from django.apps import AppConfig
from django.conf import settings
from ..handlers import MonitoringHandler
from ..client import MonitorClient
from ..config import MonitorConfig


class DjangoAIOpsHandler(MonitoringHandler):
    """
    专门为 Django LOGGING 字典配置设计的 Handler。
    它不需要在 __init__ 中传入 client，而是会自动从 settings 中拉取配置。
    """
    def __init__(self, *args, **kwargs):
        # 1. 延迟获取配置
        monitor_settings = getattr(settings, 'AIOPS_MONITOR', {})
        if not monitor_settings:
            # 如果没配置，创建一个禁用的配置，防止报错
            super().__init__(client=None, config=None, level=logging.ERROR)
            return

        # 2. 初始化配置和客户端
        try:
            config = MonitorConfig.from_dict(monitor_settings)
            client = MonitorClient(config)
            
            # 3. 初始化父类
            super().__init__(client=client, config=config)
        except Exception as e:
            # 打印到控制台，不影响主进程启动
            print(f"!!! AI-Ops Monitor 初始化异常: {e}")
            super().__init__(client=None, config=None, level=logging.ERROR)


class AIOpsMonitorConfig(AppConfig):
    """Django App 配置"""
    
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'aiops_monitor.integrations.django'
    verbose_name = 'AI-Ops Monitor'
    
    def ready(self):
        """应用启动时初始化监控（双重保险）"""
        # 如果用户已经在 LOGGING 中配置了 DjangoAIOpsHandler，则不需要自动注册
        # 这里保留一个简单的自动注册逻辑，作为默认行为
        monitor_config = getattr(settings, 'AIOPS_MONITOR', {})
        if not monitor_config:
            return
            
        # 检查是否已经手动配置了 handler
        # 我们这里不做复杂的检查，只是作为双重保险
        pass


# 默认配置
default_app_config = 'aiops_monitor.integrations.django.AIOpsMonitorConfig'
